'use strict';

var _ = require('lodash');
var sinon = require('sinon');
var React = require('react');
var PropTypes = React.PropTypes;

// TODO: Options API

var options = {
  required: true
};

var wrapPropTypes = function wrapPropTypes() {
  // Adds a .type key which allows the type to be derived during the
  // evaluation process. This is necessary for complex types which
  // return the result of a generator function, leaving no way to
  // determine which type instantiated it.

  var original = _.cloneDeep(PropTypes);

  _.each(PropTypes, function (v, k) {
    if (v.isRequired !== undefined) {
      // Simple type. Just extend the object
      _.defaultsDeep(PropTypes[k], { type: k, isRequired: { type: k } });
    } else {
      // Complex type. Must extend the creator's return value
      PropTypes[k] = function (arg) {
        return _.defaultsDeep(original[k](arg), {
          type: k, arg: arg,
          isRequired: { type: k, arg: arg }
        });
      };
    }
  });
};

wrapPropTypes();

var GENERATORS = {
  // Simple types
  array: function array() {
    return [];
  },
  bool: function bool() {
    return true;
  },
  func: function func() {
    return sinon.spy();
  },
  number: function number() {
    return 1;
  },
  object: function object() {
    return {};
  },
  string: function string() {
    return 'A String';
  },
  any: function any() {
    return 'Any';
  },
  element: function element() {
    return React.createElement('div');
  },
  node: function node() {
    return [React.createElement('div'), React.createElement('div')];
  },

  // Complex types
  arrayOf: function arrayOf(type) {
    return [generateOneProp(type)];
  },
  instanceOf: function instanceOf(klass) {
    return new klass();
  },
  objectOf: function objectOf(type) {
    return { key: generateOneProp(type) };
  },
  oneOf: function oneOf(values) {
    return _.sample(values);
  },
  oneOfType: function oneOfType(types) {
    return generateOneProp(_.extend(_.sample(types), { forceGeneration: true }));
  },
  shape: function shape(_shape) {
    return generateProps(_shape);
  }
};

var shouldGenerate = function shouldGenerate(propType) {
  return propType.forceGeneration ||
  // Generate required props, and this is the required version
  options.required && !propType.isRequired ||
  // Generate optional props, and this is the optional version
  options.optional && !!propType.isRequired;
};

var generateOneProp = function generateOneProp(propType, propName) {
  var generate = GENERATORS[propType.type];
  var arg = propType.arg;
  if (generate) {
    if (shouldGenerate(propType)) {
      if (propName) {
        return [propName, generate(arg)];
      } else {
        return generate(arg);
      }
    }
  }
};

var generateProps = function generateProps(arg) {
  var propTypes = void 0;

  if (!arg) {
    throw new TypeError('generateProps expected a propType object or a React Component');
  } else if (_.isPlainObject(arg.propTypes)) {
    propTypes = arg.propTypes;
  } else if (_.isPlainObject(arg)) {
    propTypes = arg;
  } else {
    throw new TypeError('generateProps expected a propType object or a React Component');
  }

  return _(propTypes).map(generateOneProp).compact().fromPairs().value();
};

module.exports = generateProps;